from django.contrib.auth.models import User

from is_core.main import UIRESTModelISCore

from issue_tracker.models import Issue
from issue_tracker.forms import UserForm

from .resources import NumberOfUserIssuesResource


class UserIsCore(UIRESTModelISCore):
    model = User
    form_class = UserForm
    ui_list_fields = ('id', '_obj_name')

    def has_read_permission(self, request, obj=None):
        return request.user.is_superuser or not obj or obj.pk == request.user.pk

    def has_create_permission(self, request, obj=None):
        return request.user.is_superuser

    def has_update_permission(self, request, obj=None):
        return (obj and obj.pk == request.user.pk) or request.user.is_superuser

    def has_delete_permission(self, request, obj=None):
        return request.user.is_superuser

    def get_queryset(self, request):
        qs = super(UserIsCore, self).get_queryset(request)
        if not request.user.is_superuser:
            qs = qs.filter(pk=request.user.pk)
        return qs

    def get_rest_patterns(self):
        rest_patterns = super(UserIsCore, self).get_rest_patterns()
        rest_patterns['api-user-issue'] = self.default_rest_pattern_class(
            'api-number-issues', self.site_name, r'(?P<pk>[-\w]+)/issue-number/', NumberOfUserIssuesResource, self)
        return rest_patterns


class IssueIsCore(UIRESTModelISCore):
    model = Issue
    ui_list_fields = ('id', '_obj_name', 'watched_by_string', 'leader__email', 'leader__last_name')

    def has_rest_create_permission(self, request, obj=None, via=None):
        return bool(via) and via[0].model == User



from dj.settings.base import *

DEBUG = TEMPLATE_DEBUG = THUMBNAIL_DEBUG = True

ALLOWED_HOSTS = ['localhost']

# URL with protocol (and port)
PROJECT_URL = 'localhost:8000'

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(PROJECT_DIR, 'var', 'db', 'sqlite.db'),
        'USER': '',
        'PASSWORD': '',
    },
}

STATIC_ROOT = ''

# Additional locations of static files
STATICFILES_DIRS = (
    STATICFILES_ROOT,
)



def short_description(description):
    """Sets 'short_description' attribute (this attribute is used by list_display and formular)."""
    def decorator(func):
        if isinstance(func, property):
            func = func.fget
        func.short_description = description
        return func
    return decorator



from django.utils.translation import ugettext_lazy as _

from chamber.utils.forms import formset_has_file_field

from is_core.forms.models import BaseInlineFormSet, smartinlineformset_factory, SmartModelForm
from is_core.generic_views.inlines import InlineView
from is_core.forms.fields import SmartReadonlyField, EmptyReadonlyField
from is_core.utils import get_readonly_field_data


class InlineFormView(InlineView):
    form_class = SmartModelForm
    base_inline_formset_class = BaseInlineFormSet

    model = None
    fields = None
    exclude = ()
    inline_views = None
    field_labels = None

    template_name = None

    fk_name = None
    extra = 0
    can_add = True
    can_delete = True
    is_readonly = False
    max_num = None
    min_num = 0
    readonly_fields = ()
    initial = []
    no_items_text = _('There are no items')
    class_names = ['inline-js']

    def __init__(self, request, parent_view, parent_instance):
        super(InlineFormView, self).__init__(request, parent_view, parent_instance)
        self.parent_model = parent_view.model
        self.core = parent_view.core
        self.parent_instance = parent_instance
        self.readonly = self._is_readonly()
        self.formset = self.get_formset(parent_instance, self.request.POST, self.request.FILES)

        for i in range(self.get_min_num()):
            self.formset.forms[i].empty_permitted = False

    def _get_field_labels(self):
        return self.field_labels

    def _is_readonly(self):
        return self.is_readonly or not self.parent_view.has_post_permission(obj=self.parent_view.get_obj())

    def can_form_delete(self, form):
        return not self.is_form_readonly(form)

    def is_form_readonly(self, form):
        return self.readonly

    def get_context_data(self, **kwargs):
        context_data = super(InlineFormView, self).get_context_data(**kwargs)
        context_data.update({
            'formset': self.formset,
            'fieldset': self.get_fieldset(self.formset),
            'name': self.get_name(),
            'button_value': self.get_button_value(),
            'class_names': self.get_class_names(**kwargs),
            'no_items_text': self.no_items_text
        })
        return context_data

    def get_class_names(self, **kwargs):
        class_names = self.class_names + [self.get_name().lower()]
        if self.formset.can_add:
            class_names.append('can-add')
        if self.formset.can_delete:
            class_names.append('can-delete')

        if kwargs.get('title'):
            class_names.append('with-title')
        else:
            class_names.append('without-title')

        return class_names

    def get_exclude(self):
        return self.exclude

    def get_fields(self):
        return self.fields

    def get_extra(self):
        return self.extra + len(self.get_initial())

    def get_initial(self):
        return self.initial[:]

    def get_can_delete(self):
        return self.can_delete and not self.readonly

    def get_can_add(self):
        return self.can_add and not self.readonly

    def get_readonly_fields(self):
        return self.readonly_fields

    def get_prefix(self):
        return '-'.join((self.parent_view.get_prefix(), 'inline', self.__class__.__name__)).lower()

    def get_fieldset(self, formset):
        fields = list(self.get_fields() or formset.form.base_fields.keys())
        if self.get_can_delete():
            fields.append('DELETE')
        return fields

    def formfield_for_dbfield(self, db_field, **kwargs):
        return db_field.formfield(**kwargs)

    def formfield_for_readonlyfield(self, name, **kwargs):
        def _get_readonly_field_data(instance):
            return get_readonly_field_data(name, (self, self.core, instance), {'request': self.request})
        return SmartReadonlyField(_get_readonly_field_data)

    def get_form_class(self):
        return self.form_class

    def get_max_num(self):
        return self.max_num

    def get_min_num(self):
        return self.min_num

    def get_formset_factory(self, fields=None, readonly_fields=()):
        return smartinlineformset_factory(
            self.parent_model, self.model, self.request, form=self.get_form_class(), fk_name=self.fk_name,
            extra=self.get_extra(), formset=self.base_inline_formset_class, can_delete=self.get_can_delete(),
            exclude=self.get_exclude(), fields=fields, min_num=self.get_min_num(), max_num=self.get_max_num(),
            readonly_fields=readonly_fields, readonly=self.readonly,
            formreadonlyfield_callback=self.formfield_for_readonlyfield, formfield_callback=self.formfield_for_dbfield,
            labels=self._get_field_labels()
        )

    def get_queryset(self):
        return self.model.objects.all()

    def get_formset(self, instance, data, files):
        fields = self.get_fields()
        readonly_fields = self.get_readonly_fields()

        if data:
            formset = self.get_formset_factory(fields, readonly_fields)(data=data, files=files, instance=instance,
                                                                        queryset=self.get_queryset(),
                                                                        prefix=self.get_prefix())
        else:
            formset = self.get_formset_factory(fields, readonly_fields)(instance=instance,
                                                                        queryset=self.get_queryset(),
                                                                        initial=self.get_initial(),
                                                                        prefix=self.get_prefix())
        formset.can_add = self.get_can_add()
        formset.can_delete = self.get_can_delete()

        for form in formset.all_forms():
            form.class_names = self.form_class_names(form)
            form._is_readonly = self.is_form_readonly(form)
            if form._is_readonly and not self.readonly:
                if not formset.can_delete:
                    form.readonly_fields = set(form.fields.keys()) - {'id'}
                elif not self.can_form_delete(form):
                    form.readonly_fields = set(form.fields.keys()) - {'id'}
                    form.fields['DELETE'] = EmptyReadonlyField()
                else:
                    form.readonly_fields = set(form.fields.keys()) - {'id', 'DELETE'}
            elif not self.can_form_delete(form):
                form.fields['DELETE'] = EmptyReadonlyField()
                form.readonly_fields |= {'DELETE'}
            self.init_form(form)
        return formset

    def form_class_names(self, form):
        if not form.instance.pk:
            return ['empty']
        return []

    def init_form(self, form):
        self.form_fields(form)

    def form_fields(self, form):
        for field_name, field in form.fields.items():
            field = self.form_field(form, field_name, field)

    def form_field(self, form, field_name, form_field):
        placeholder = self.model._ui_meta.placeholders.get('field_name', None)
        if placeholder:
            form_field.widget.placeholder = self.model._ui_meta.placeholders.get('field_name', None)
        return form_field

    def get_name(self):
        return self.model.__name__

    def get_button_value(self):
        return (self.model._ui_meta.add_inline_button_verbose_name % {
            'verbose_name': self.model._meta.verbose_name,
            'verbose_name_plural': self.model._meta.verbose_name_plural
        })

    def form_valid(self, request):
        instances = self.formset.save(commit=False)
        for obj in instances:
            change = obj.pk is not None
            self.save_obj(obj, change)
        for obj in self.formset.deleted_objects:
            self.delete_obj(obj)
        self.formset.save_m2m()

    def get_has_file_field(self):
        return formset_has_file_field(self.formset.form)

    def pre_save_obj(self, obj, change):
        pass

    def post_save_obj(self, obj, change):
        pass

    def save_obj(self, obj, change):
        self.pre_save_obj(obj, change)
        obj.save()
        self.post_save_obj(obj, change)

    def pre_delete_obj(self, obj):
        pass

    def post_delete_obj(self, obj):
        pass

    def delete_obj(self, obj):
        self.pre_delete_obj(obj)
        obj.delete()
        self.post_delete_obj(obj)

    def is_valid(self):
        return self.formset.is_valid()


class TabularInlineFormView(InlineFormView):
    template_name = 'forms/tabular_inline_formset.html'


class StackedInlineFormView(InlineFormView):
    template_name = 'forms/stacked_inline_formset.html'


class ResponsiveInlineFormView(InlineFormView):
    template_name = 'forms/responsive_inline_formset.html'



from is_core.tests.factory.fields import delete_test_files


class DataGeneratorTestCase:

    factories = {}

    @classmethod
    def tearDownClass(cls):
        super(DataGeneratorTestCase, cls).tearDownClass()
        delete_test_files()

    @classmethod
    def get_model_label(cls, model):
        return '%s.%s' % (model._meta.app_label, model._meta.object_name)

    def new_instance(self, model, only_build=False):
        model_label = self.get_model_label(model)
        if model_label in self.factories:
            if only_build:
                return self.factories.get(model_label).build()
            else:
                return self.factories.get(model_label)()



class PermissionsMixin:
    """
    Mixin that validate user permissions inside ISCore
    """

    read_permission = True
    create_permission = True
    update_permission = True
    delete_permission = True

    def has_read_permission(self, request, obj=None):
        return self.read_permission

    def has_create_permission(self, request, obj=None):
        return self.create_permission

    def has_update_permission(self, request, obj=None):
        return self.update_permission

    def has_delete_permission(self, request, obj=None):
        return self.delete_permission


class PermissionsUIMixin(PermissionsMixin):
    """
    Mixin that validate UI user permissions inside ISCore
    """

    ui_read_permission = None
    ui_create_permission = None
    ui_update_permission = None
    ui_delete_permission = None

    def has_ui_read_permission(self, request, obj=None):
        return (
            self.has_read_permission(request, obj) if self.ui_read_permission is None else self.ui_read_permission
        )

    def has_ui_create_permission(self, request, obj=None):
        return (
            self.has_create_permission(request, obj) if self.ui_create_permission is None else self.ui_create_permission
        )

    def has_ui_update_permission(self, request, obj=None):
        return (
            self.has_update_permission(request, obj) if self.ui_update_permission is None else self.ui_update_permission
        )

    def has_ui_delete_permission(self, request, obj=None):
        return (
            self.has_delete_permission(request, obj) if self.ui_delete_permission is None else self.ui_delete_permission
        )


class PermissionsRESTMixin(PermissionsMixin):
    """
    Mixin that validate REST user permissions inside ISCore
    """

    rest_read_permission = None
    rest_create_permission = None
    rest_update_permission = None
    rest_delete_permission = None

    def has_rest_read_permission(self, request, obj=None, via=None):
        return (
            self.has_read_permission(request, obj)
            if self.rest_read_permission is None else self.rest_read_permission
        )

    def has_rest_create_permission(self, request, obj=None, via=None):
        return (
            self.has_create_permission(request, obj)
            if self.rest_create_permission is None else self.rest_create_permission
        )

    def has_rest_update_permission(self, request, obj=None, via=None):
        return (
            self.has_update_permission(request, obj)
            if self.rest_update_permission is None else self.rest_update_permission
        )

    def has_rest_delete_permission(self, request, obj=None, via=None):
        return (
            self.has_delete_permission(request, obj)
            if self.rest_delete_permission is None else self.rest_delete_permission
        )



from django import template

from is_core.config import settings
from is_core.site import get_site_by_name
from is_core.utils import str_to_class

register = template.Library()


class MenuItemPattern():

    def __init__(self, title, pattern, show_in_menu=True, submenu_items=[], pattern_kwargs={}):
        self.title = title
        self.pattern = pattern
        self.show_in_menu = show_in_menu
        self.pattern_kwargs = pattern_kwargs


@register.inclusion_tag('menu/sub_menu.html', takes_context=True)
def submenu(context, menu_items):
    return {'menu_items': menu_items}


@register.inclusion_tag('menu/menu.html', takes_context=True)
def menu(context, site_name):
    site = get_site_by_name(site_name)
    request = context.get('request')

    active_menu_groups = context.get('active_menu_groups')

    menu_generator = str_to_class(settings.MENU_GENERATOR)(request, site, active_menu_groups)
    menu_items = menu_generator.get_menu_items(menu_generator.get_menu_structure())
    context.update({'menu_items': menu_items, 'site_name': site_name})
    return context


@register.inclusion_tag('menu/bread_crumbs.html', takes_context=True)
def bread_crumbs(context):
    bread_crumbs_menu_items = context.get('bread_crumbs_menu_items')

    menu_items = []

    for menu_item in bread_crumbs_menu_items or []:
        menu_items.append(menu_item)
    return {'menu_items': menu_items}



from django.utils.functional import cached_property

from is_core.patterns import reverse_pattern

from .exceptions import GenericViewException


class ListParentMixin:

    def list_bread_crumbs_menu_item(self):
        from is_core.menu import LinkMenuItem

        list_pattern = self.core.ui_patterns.get('list')
        list_view = list_pattern.get_view(self.request)
        return LinkMenuItem(self.model._ui_meta.list_verbose_name %
                            {'verbose_name': list_view.model._meta.verbose_name,
                             'verbose_name_plural': list_view.model._meta.verbose_name_plural},
                             list_pattern.get_url_string(self.request),
                             active=not self.add_current_to_breadcrumbs)

    def parent_bread_crumbs_menu_items(self):
        menu_items = []
        if ('list' in self.core.ui_patterns and
                self.core.ui_patterns.get('list').get_view(self.request).has_get_permission()):
            menu_items.append(self.list_bread_crumbs_menu_item())
        return menu_items

    def bread_crumbs_menu_items(self):
        return self.parent_bread_crumbs_menu_items() + super(ListParentMixin, self).bread_crumbs_menu_items()


class EditParentMixin(ListParentMixin):

    def edit_bread_crumbs_menu_item(self):
        from is_core.menu import LinkMenuItem

        edit_pattern = self.core.ui_patterns.get('edit')
        edit_view = edit_pattern.get_view(self.request)
        parent_obj = self.get_parent_obj()
        if not isinstance(parent_obj, edit_view.model):
            raise GenericViewException('Parent obj must be instance of edit view model')

        return LinkMenuItem(self.model._ui_meta.edit_verbose_name %
                            {'verbose_name': edit_view.model._meta.verbose_name,
                             'verbose_name_plural': edit_view.model._meta.verbose_name_plural,
                             'obj': parent_obj},
                             edit_pattern.get_url_string(self.request, kwargs={'pk':parent_obj.pk}),
                             active=not self.add_current_to_breadcrumbs)

    def parent_bread_crumbs_menu_items(self):
        menu_items = super(EditParentMixin, self).parent_bread_crumbs_menu_items()
        if 'edit' in self.core.ui_patterns \
                and self.core.ui_patterns.get('edit').get_view(self.request).has_get_permission(obj=self.get_obj()):
            menu_items.append(self.edit_bread_crumbs_menu_item())
        return menu_items

    def get_parent_obj(self):
        return self.get_obj()


class TabItem:

    def __init__(self, pattern_name, label, can_show=None, is_active=None, pattern_kwargs=None,
                 is_active_starts_with=True):
        self._pattern_name = pattern_name
        self._label = label
        self._is_active = is_active
        self._pattern_kwargs = pattern_kwargs
        self._can_show = can_show
        self._is_active_starts_with = is_active_starts_with

    @cached_property
    def pattern(self):
        return reverse_pattern(self._pattern_name)

    def get_pattern_kwargs(self, view):
        pattern_kwargs = self._pattern_kwargs or {}
        return pattern_kwargs(view) if hasattr(pattern_kwargs, '__call__') else pattern_kwargs

    def can_show(self, view):
        can_show = self._can_show
        if can_show is not None:
            return can_show(view) if hasattr(can_show, '__call__') else can_show
        else:
            return self.pattern.can_call_get(view.request, **self.get_pattern_kwargs(view))

    def get_url(self, view):
        return self.pattern.get_url_string(view.request, kwargs=self.get_pattern_kwargs(view))

    def is_active(self, view):
        is_active = self._is_active
        if is_active is not None:
            return is_active(view) if hasattr(is_active, '__call__') else is_active
        else:
            return (
                (self._is_active_starts_with and view.request.path.startswith(self.get_url(view))) or
                (not self._is_active_starts_with and view.request.path == self.get_url(view))
            )

    def get_menu_link_item_or_none(self, view):
        from is_core.menu import LinkMenuItem

        return LinkMenuItem(
            self._label, self.get_url(view), active=self.is_active(view)) if self.can_show(view) else None


class TabsViewMixin:

    tabs = ()

    def get_tabs(self):
        return self.tabs

    def get_tab_menu_items(self):
        return [tab.get_menu_link_item_or_none(self) for tab in self.get_tabs() if tab.can_show(self)]

    def get_context_data(self, **kwargs):
        context_data = super(TabsViewMixin, self).get_context_data(**kwargs)
        context_data.update({
            'tabs': self.get_tab_menu_items(),
        })
        return context_data


class GetCoreObjViewMixin:
    pk_name = 'pk'

    def get_obj_filters(self):
        filters = {'pk': self.kwargs.get(self.pk_name)}
        return filters

    # TODO: should contains own implementation (not use get_obj from main)
    _obj = None
    def get_obj(self, cached=True):
        if cached and self._obj:
            return self._obj
        obj = self.core.get_obj(self.request, **self.get_obj_filters())
        if cached and not self._obj:
            self._obj = obj
        return obj



from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _

from is_core.exceptions.response import response_exception_factory


def throttling_failure_view(request, exception):
    return response_exception_factory(request, 429, _('Too Many Requests'), force_text(exception))



from germanium.test_cases.rest import RESTTestCase
from germanium.annotations import login
from germanium.tools import assert_in
from germanium.tools.http import assert_http_unauthorized, assert_http_forbidden, assert_http_not_found

from .test_case import HelperTestCase, AsSuperuserTestCase


class HttpExceptionsTestCase(AsSuperuserTestCase, HelperTestCase, RESTTestCase):
    ISSUE_API_URL = '/api/issue/'
    USER_API_URL = '/api/user/'
    ACCEPT_TYPES = ('application/json', 'text/xml', 'text/csv',
                    'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet')

    def test_401_exception(self):
        for accept_type in self.ACCEPT_TYPES:
            resp = self.get(self.ISSUE_API_URL, headers={'HTTP_ACCEPT': accept_type})
            assert_in(accept_type, resp['Content-Type'])
            assert_http_unauthorized(resp)

    @login(is_superuser=False)
    def test_403_exception(self):
        user = self.get_user_obj()
        for accept_type in self.ACCEPT_TYPES:
            resp = self.get('%s%s/' % (self.USER_API_URL, user.pk), headers={'HTTP_ACCEPT': accept_type})
            assert_in(accept_type, resp['Content-Type'])
            assert_http_forbidden(resp)

    @login(is_superuser=False)
    def test_404_exception(self):
        for accept_type in self.ACCEPT_TYPES:
            resp = self.get('%s%s/' % (self.ISSUE_API_URL, 5), headers={'HTTP_ACCEPT': accept_type})
            assert_in(accept_type, resp['Content-Type'])
            assert_http_not_found(resp)

    @login(is_superuser=True)
    def test_403_csrf_exception(self):
        self.c = self.client_class(enforce_csrf_checks=True)
        for accept_type in self.ACCEPT_TYPES:
            resp = self.post(self.ISSUE_API_URL, {}, headers={'HTTP_ACCEPT': accept_type,
                                                                              'CONTENT_TYPE': 'application/json'})
            assert_in(accept_type, resp['Content-Type'])
            assert_http_forbidden(resp)



from django.contrib.auth.models import User

from germanium import config
from germanium.annotations import login
from germanium.test_cases.client import ClientTestCase
from germanium.tools import assert_true, assert_false, assert_equal, assert_not_equal
from germanium.tools.http import assert_http_redirect, assert_http_ok, assert_http_forbidden

from .test_case import HelperTestCase, AsSuperuserTestCase


class UIPermissionsTestCase(AsSuperuserTestCase, HelperTestCase, ClientTestCase):
    USER_UI_URL = '/user/'

    def authorize(self, username, password):
        resp = self.post(config.LOGIN_URL, {config.USERNAME: username, config.PASSWORD: password})
        assert_http_redirect(resp)

    def test_non_logged_user_should_receive_302(self):
        resp = self.get(self.USER_UI_URL)
        assert_http_redirect(resp)

    @login(is_superuser=False)
    def test_home_view_should_return_ok(self):
        resp = self.get('/')
        assert_http_ok(resp)

    @login(is_superuser=True)
    def test_superuser_may_read_users_grid(self):
        resp = self.get(self.USER_UI_URL)
        assert_http_ok(resp)

    @login(is_superuser=False)
    def test_ouser_can_read_users_grid(self):
        resp = self.get(self.USER_UI_URL)
        assert_http_ok(resp)

    @login(is_superuser=True)
    def test_superuser_may_edit_user(self):
        user = self.get_user_obj()
        resp = self.get('%s%s/' % (self.USER_UI_URL, user.pk))
        assert_http_ok(resp)

        CHANGED_USERNAME = 'changed_nick'
        self.post('%s%s/' % (self.USER_UI_URL, user.pk), data={'edit-is-user-username': CHANGED_USERNAME})
        assert_http_ok(resp)
        assert_equal(User.objects.get(pk=user.pk).username, CHANGED_USERNAME)

    @login(is_superuser=False)
    def test_only_superuser_may_edit_user(self):
        user = self.get_user_obj()
        resp = self.get('%s%s/' % (self.USER_UI_URL, user.pk))
        assert_http_forbidden(resp)

        CHANGED_USERNAME = 'changed_nick'
        self.post('%s%s/' % (self.USER_UI_URL, user.pk), data={'edit-is-user-username': CHANGED_USERNAME})
        assert_http_forbidden(resp)
        assert_not_equal(User.objects.get(pk=user.pk).username, CHANGED_USERNAME)

    @login(is_superuser=False)
    def test_user_may_edit_itself(self):
        user = self.logged_user.user
        resp = self.get('%s%s/' % (self.USER_UI_URL, user.pk))
        assert_http_ok(resp)

        CHANGED_USERNAME = 'changed_nick'
        self.post('%s%s/' % (self.USER_UI_URL, user.pk), data={'edit-is-user-username': CHANGED_USERNAME})
        assert_http_ok(resp)
        assert_equal(User.objects.get(pk=user.pk).username, CHANGED_USERNAME)

    @login(is_superuser=True)
    def test_superuser_may_add_user(self):
        USERNAME = 'new_nick'

        resp = self.post('%sadd/' % self.USER_UI_URL, data={'add-is-user-username': USERNAME,
                                                            'add-is-user-password': 'password'})
        assert_http_redirect(resp)
        assert_true(User.objects.filter(username=USERNAME).exists())

    @login(is_superuser=False)
    def test_only_superuser_may_add_user(self):
        USERNAME = 'new_nick'

        resp = self.post('%sadd/' % self.USER_UI_URL, data={'add-is-user-username': USERNAME,
                                                            'add-is-user-password': 'password'})
        assert_http_forbidden(resp)
        assert_false(User.objects.filter(username=USERNAME).exists())



from django.conf import settings as django_settings


DEFAULTS = {
    'AUTH_FORM_CLASS': (
        'auth_token.contrib.is_core_auth.forms.TokenAuthenticationSmartForm'
        if 'auth_token' in django_settings.INSTALLED_APPS
        else 'django.contrib.auth.forms.AuthenticationForm'
    ),
    'AUTH_RESOURCE_CLASS': (
        'auth_token.contrib.is_core_auth.resource.AuthResource' if 'auth_token' in django_settings.INSTALLED_APPS
        else None
    ),
    'AUTH_LOGIN_VIEW': (
        'auth_token.contrib.is_core_auth.views.TokenLoginView' if 'auth_token' in django_settings.INSTALLED_APPS
        else 'django.contrib.auth.views.LoginView'
    ),
    'AUTH_LOGOUT_VIEW': (
        'auth_token.contrib.is_core_auth.views.TokenLogoutView' if 'auth_token' in django_settings.INSTALLED_APPS
        else 'django.contrib.auth.views.LogoutView'
    ),
    'HOME_CORE': 'is_core.main.HomeUIISCore',
    'HOME_VIEW': 'is_core.generic_views.HomeView',
    'MENU_GENERATOR': 'is_core.menu.MenuGenerator',
    'USERNAME': 'username',
    'PASSWORD': 'password',
    'LOGIN_URL': django_settings.LOGIN_URL,
    'LOGOUT_URL': '/logout/',
    'LOGIN_API_URL': lambda s: '/api{}'.format(s.LOGIN_URL),
    'EXPORT_TYPES': '',
    'FOREIGN_KEY_MAX_SELECBOX_ENTRIES': 500,
    'LIST_PER_PAGE': 20,
    'REST_DEFAULT_FIELDS_EXTENSION': ('_rest_links',),
    'RESPONSE_EXCEPTION_FACTORY': 'is_core.exceptions.response.ui_rest_response_exception_factory',
}


class Settings:

    def __getattr__(self, attr):
        if attr not in DEFAULTS:
            raise AttributeError('Invalid IS_CORE setting: "{}"'.format(attr))

        default = DEFAULTS[attr]
        return getattr(django_settings, 'IS_CORE_{}'.format(attr), default(self) if callable(default) else default)


settings = Settings()
